(function ($, Drupal, once) {
  'use strict';

  /**
   * Update term position via AJAX.
   */
  function updateTermPosition(termId, parentId, weight, oldParentId) {
    // First, get a fresh CSRF token
    $.ajax({
      url: '/session/token',
      method: 'GET',
    }).done(function (csrfToken) {
      // Log the request details
      console.log('Updating term position:', {
        termId: termId,
        parentId: parentId,
        weight: weight,
      });

      $.ajax({
        url: `/admin/structure/taxonomy/term/${termId}/update-tree`,
        method: 'POST',
        headers: {
          'X-CSRF-Token': csrfToken,
          'Content-Type': 'application/json',
        },
        data: JSON.stringify({
          parent_tid: parentId,
          weight: weight,
        }),
        success: function (response) {
          console.log('Server response:', response);
          if (response.status === 'success') {
            console.log('Term update successful');
          } else {
            console.error('Server returned error:', response);
            alert(
              'Failed to update term position: ' +
                (response.message || 'Unknown error')
            );
          }
        },
        error: function (xhr, status, error) {
          console.error('Term update AJAX error:', {
            status: status,
            error: error,
            responseText: xhr.responseText,
            statusCode: xhr.status,
            statusText: xhr.statusText,
          });

          let errorMessage = 'Failed to update term position.';
          try {
            const response = JSON.parse(xhr.responseText);
            if (response.message) { 
              errorMessage += ' ' + response.message;
            }
          } catch (e) {
            console.log('Could not parse error response:', xhr.responseText);
          }

          alert(
            errorMessage + ' Please check the browser console for more details.'
          );
        },
      });
    });
  }

  /**
   * Initialize Sortable on a container.
   */
  function initializeSortable(container) {
    return new Sortable(container, {
      group: 'taxonomy-terms',
      animation: 150,
      handle: '.term-handle',
      ghostClass: 'sortable-ghost',
      chosenClass: 'sortable-chosen',
      onEnd: function (evt) {
        const $item = $(evt.item);
        const $newParent = $(evt.to).closest('.taxonomy-term');
        const $oldParent = $(evt.from).closest('.taxonomy-term');
        const termId = $item.data('tid');
        const newParentId = $newParent.length ? $newParent.data('tid') : 0;
        const oldParentId = $oldParent.length ? $oldParent.data('tid') : 0;
        const weight = evt.newIndex;
        const termName = $item.find('.term-label').text();

        // If parent has changed, show confirmation
        if (newParentId !== oldParentId) {
          const oldParentName = oldParentId
            ? $oldParent.find('> .term-content .term-label').text()
            : 'root level';
          const newParentName = newParentId
            ? $newParent.find('> .term-content .term-label').text()
            : 'root level';

          const message = `Are you sure you want to move "${termName}" from "${oldParentName}" to "${newParentName}"?`;

          if (!confirm(message)) {
            // Revert the move
            if (evt.from !== evt.to) {
              evt.from.insertBefore(evt.item, evt.from.children[evt.oldIndex]);
            }
            return;
          }
        }

        // Add highlight effect
        $item.addClass('highlight-success');
        setTimeout(function () {
          $item.removeClass('highlight-success');
        }, 1000);

        updateTermPosition(termId, newParentId, weight, oldParentId);
      },
    });
  }

  /**
   * Create term HTML structure
   */
  function createTermHtml(child) {
    return $(`
      <li class="taxonomy-term"
          data-tid="${child.tid}"
          data-weight="${child.weight}"
          data-depth="${child.depth}">
        <div class="term-content">
          ${child.hasChildren ? '<span class="toggle-children collapsed"></span>' : ''}
          <span class="term-handle"></span>
          <span class="term-label">${child.name}</span>
          <div class="term-operations">
            <button class="add-term">Add Term</button>
            <button class="edit-term">Edit</button>
            <button class="delete-term">Delete</button>
          </div>
        </div>
        ${child.hasChildren ? '<ul class="child-terms"></ul>' : ''}
      </li>
    `);
  }

  function addTerm(vocabularyId, parentId = null) {
    const element = document.createElement('div');
    const ajaxSettings = {
      url: Drupal.url(
        `admin/structure/taxonomy/manage/${vocabularyId}/add-term`
      ),
      dialogType: 'modal',
      dialog: {
        width: 'auto',
        title: Drupal.t('Add term'),
        dialogClass: 'taxonomy-term-form-dialog',
      },
    };

    const ajax = new Drupal.Ajax(false, element, ajaxSettings);
    ajax.execute().then(function () {
      // After modal is opened, set the parent field if provided
      if (parentId) {
        setTimeout(function () {
          const $parentField = $(
            '.taxonomy-term-form-dialog select[name="parent[]"]'
          );
          if ($parentField.length) {
            $parentField.val([parentId]);
          }
        }, 300);
      }
    });
  }

  Drupal.behaviors.taxonomyManager = {
    attach: function (context, settings) {
      // Initialize Sortable on the main tree and all child-terms containers
      once('taxonomy-manager', '.taxonomy-tree, .child-terms', context).forEach(
        function (container) {
          initializeSortable(container);
        }
      );

      // Search functionality
      once('taxonomy-manager-search', '.taxonomy-search', context).forEach(
        function (search) {
          $(search).on('input', function () {
            const searchTerm = $(this).val().toLowerCase();

            $('.taxonomy-term').each(function () {
              const $term = $(this);
              const termName = $term.find('.term-label').text().toLowerCase();

              if (termName.includes(searchTerm)) {
                $term.show();
                // Show all parent terms
                $term.parents('.taxonomy-term').show();
              } else {
                $term.hide();
              }
            });
          });
        }
      );

      // Expand/Collapse all
      once('taxonomy-manager-expand-all', '.expand-all', context).forEach(
        function (button) {
          $(button).on('click', function () {
            $('.toggle-children.collapsed').trigger('click');
          });
        }
      );

      once('taxonomy-manager-collapse-all', '.collapse-all', context).forEach(
        function (button) {
          $(button).on('click', function () {
            $('.toggle-children.expanded').trigger('click');
          });
        }
      );

      // Handle toggle clicks using event delegation
      once('taxonomy-manager-toggle', '.taxonomy-tree', context).forEach(
        function (tree) {
          $(tree).on('click', '.toggle-children', function (event) {
            event.preventDefault();
            event.stopPropagation();

            const $toggle = $(event.currentTarget);
            const $term = $toggle.closest('.taxonomy-term');
            const $children = $term.find('> .child-terms');
            const isCollapsed = $toggle.hasClass('collapsed');

            // Prevent multiple simultaneous toggles
            if ($toggle.data('processing')) {
              return;
            }
            $toggle.data('processing', true);

            function finishToggle(expand) {
              if (expand) {
                $children.slideDown(400, function () {
                  $toggle.removeClass('collapsed').addClass('expanded');
                  $toggle.data('processing', false);
                });
              } else {
                $children.slideUp(400, function () {
                  $toggle.removeClass('expanded').addClass('collapsed');
                  $toggle.data('processing', false);
                });
              }
            }

            if (isCollapsed) {
              // Load children if not already loaded
              if ($children.is(':empty')) {
                const termId = $term.data('tid');
                $.get(
                  Drupal.url(
                    `admin/structure/taxonomy/term/${termId}/children`
                  ),
                  function (data) {
                    // Create child container if it doesn't exist
                    if ($children.length === 0) {
                      $children = $(
                        '<ul class="child-terms" style="display: none;"></ul>'
                      ).appendTo($term);
                    }

                    // Render children using the createTermHtml function
                    data.forEach(function (child) {
                      const $childTerm = createTermHtml(child);
                      $children.append($childTerm);
                    });

                    // Initialize Sortable on the new child-terms container
                    initializeSortable($children[0]);

                    // Show the children
                    finishToggle(true);

                    // Attach behaviors to the new content
                    Drupal.attachBehaviors($children[0]);
                  }
                );
              } else {
                finishToggle(true);
              }
            } else {
              finishToggle(false);
            }
          });
        }
      );

      // Handle add term button clicks
      once('taxonomy-manager-add', '.add-term', context).forEach(
        function (button) {
          $(button).on('click', function (e) {
            e.preventDefault();
            const $term = $(this).closest('.taxonomy-term');
            const parentId = $term.data('tid');
            const vocabularyId = drupalSettings.taxonomy_manager.vocabularyId;
            addTerm(vocabularyId, parentId);
          });
        }
      );

      // Handle edit button clicks
      once('taxonomy-manager-edit', '.edit-term', context).forEach(
        function (button) {
          $(button).on('click', function (e) {
            e.preventDefault();
            const $term = $(this).closest('.taxonomy-term');
            const termId = $term.data('tid');

            // Create and open modal dialog
            const $modal = $('<div>').dialog({
              title: Drupal.t('Edit Term'),
              width: '80%',
              maxWidth: '800px',
              modal: true,
              close: function () {
                $(this).remove();
              },
            });

            // Load the form into the modal
            $.get(
              Drupal.url(`admin/structure/taxonomy/term/${termId}/edit-ajax`),
              function (response) {
                $modal.html(response);
                Drupal.attachBehaviors($modal[0]);
              }
            );
          });
        }
      );

      // Handle delete button clicks
      once('taxonomy-manager-delete', '.delete-term', context).forEach(
        function (button) {
          $(button).on('click', function (e) {
            e.preventDefault();
            const $term = $(this).closest('.taxonomy-term');
            const termId = $term.data('tid');

            if (
              confirm(
                Drupal.t(
                  'Are you sure you want to delete this term? This action cannot be undone.'
                )
              )
            ) {
              $.get(
                Drupal.url(
                  `admin/structure/taxonomy/term/${termId}/delete-ajax`
                ),
                function (response) {
                  // Remove the term from the UI
                  $term.fadeOut(400, function () {
                    $(this).remove();
                  });
                }
              );
            }
          });
        }
      );

      // Handle taxonomy tree refresh
      $(document).on(
        'DOMNodeInserted',
        '.taxonomy-tree-refresh-marker',
        function () {
          const $marker = $(this);
          const $term = $marker.closest('.taxonomy-term');
          const termId = $term.data('tid');

          // If we're refreshing a specific term's children
          if (termId) {
            const $children = $term.find('> .child-terms');
            const $toggle = $term.find('> .term-content .toggle-children');

            // If toggle doesn't exist, add it
            if (!$toggle.length) {
              $term
                .find('> .term-content')
                .prepend('<span class="toggle-children collapsed"></span>');
            }

            // If children container doesn't exist, add it
            if (!$children.length) {
              $term.append('<ul class="child-terms"></ul>');
            }

            // Force a refresh of the children
            $children.empty();
            const $newToggle = $term.find('> .term-content .toggle-children');
            if ($newToggle.length) {
              $newToggle.trigger('click');
            }
          } else {
            // Full tree refresh - reload the page
            window.location.reload();
          }
        }
      );
    },
  };
})(jQuery, Drupal, once);
