<?php

namespace Drupal\enhanced_taxonomy_manager\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\taxonomy\VocabularyInterface;
use Drupal\taxonomy\TermInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityFormBuilderInterface;
use Drupal\Core\Form\FormBuilderInterface;

/**
 * Controller for the taxonomy manager.
 */
class TaxonomyManagerController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity repository.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  protected $entityRepository;

  /**
   * The entity form builder.
   *
   * @var \Drupal\Core\Entity\EntityFormBuilderInterface
   */
  protected $entityFormBuilder;

  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * Constructs a TaxonomyManagerController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository
   *   The entity repository.
   * @param \Drupal\Core\Entity\EntityFormBuilderInterface $entity_form_builder
   *   The entity form builder.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    EntityRepositoryInterface $entity_repository,
    EntityFormBuilderInterface $entity_form_builder,
    FormBuilderInterface $form_builder,
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->entityRepository = $entity_repository;
    $this->entityFormBuilder = $entity_form_builder;
    $this->formBuilder = $form_builder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity.repository'),
      $container->get('entity.form_builder'),
      $container->get('form_builder')
    );
  }

  /**
   * Displays the taxonomy tree view for a vocabulary.
   *
   * @param \Drupal\taxonomy\VocabularyInterface $taxonomy_vocabulary
   *   The vocabulary to display.
   *
   * @return array
   *   A render array.
   */
  public function vocabularyOverview(VocabularyInterface $taxonomy_vocabulary) {
    $terms = $this->loadTermTree($taxonomy_vocabulary->id(), 0, 1);

    return [
      '#theme' => 'taxonomy_tree_view',
      '#vocabulary' => $taxonomy_vocabulary,
      '#terms' => $terms,
      '#attached' => [
        'library' => ['enhanced_taxonomy_manager/taxonomy_manager'],
        'drupalSettings' => [
          'taxonomy_manager' => [
            'vocabularyId' => $taxonomy_vocabulary->id(),
            'token' => \Drupal::csrfToken()->get('taxonomy_term_update_tree'),
          ],
        ],
      ],
    ];
  }

  /**
   * Updates a term's position in the taxonomy tree.
   *
   * @param \Drupal\taxonomy\TermInterface $taxonomy_term
   *   The term being updated.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response indicating success or failure.
   */
  public function updateTermPosition(TermInterface $taxonomy_term, Request $request) {
    $payload = json_decode($request->getContent(), TRUE);
    $new_parent_tid = $payload['parent_tid'] ?? 0;
    $weight = $payload['weight'] ?? 0;

    // Log the incoming request.
    \Drupal::logger('enhanced_taxonomy_manager')->notice(
      'Attempting to update term @tid: parent=@parent, weight=@weight, payload=@payload',
      [
        '@tid' => $taxonomy_term->id(),
        '@parent' => $new_parent_tid,
        '@weight' => $weight,
        '@payload' => print_r($payload, TRUE),
      ]
    );

    try {
      if ($new_parent_tid) {
        $parent = $this->entityTypeManager->getStorage('taxonomy_term')->load($new_parent_tid);
        if (!$parent) {
          \Drupal::logger('enhanced_taxonomy_manager')->error(
            'Parent term @parent_tid not found for term @tid',
            [
              '@parent_tid' => $new_parent_tid,
              '@tid' => $taxonomy_term->id(),
            ]
          );
          return new JsonResponse(['status' => 'error', 'message' => 'Parent term not found'], 404);
        }
        $taxonomy_term->set('parent', $parent->id());
        \Drupal::logger('enhanced_taxonomy_manager')->info(
          'Setting parent of term @tid to @parent',
          [
            '@tid' => $taxonomy_term->id(),
            '@parent' => $parent->id(),
          ]
        );
      }
      else {
        $taxonomy_term->set('parent', 0);
        \Drupal::logger('enhanced_taxonomy_manager')->info(
          'Setting term @tid as root level term',
          ['@tid' => $taxonomy_term->id()]
        );
      }

      $taxonomy_term->setWeight($weight);
      $taxonomy_term->save();

      \Drupal::logger('enhanced_taxonomy_manager')->notice(
        'Successfully updated term @tid',
        ['@tid' => $taxonomy_term->id()]
      );

      return new JsonResponse(['status' => 'success']);
    }
    catch (\Exception $e) {
      \Drupal::logger('enhanced_taxonomy_manager')->error(
        'Failed to update term @tid: @error',
        [
          '@tid' => $taxonomy_term->id(),
          '@error' => $e->getMessage(),
        ]
      );
      return new JsonResponse(
        [
          'status' => 'error',
          'message' => $e->getMessage(),
          'trace' => $e->getTraceAsString(),
        ],
        500
      );
    }
  }

  /**
   * Gets the children of a taxonomy term.
   *
   * @param \Drupal\taxonomy\TermInterface $taxonomy_term
   *   The parent term.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response containing the child terms.
   */
  public function getTermChildren(TermInterface $taxonomy_term) {
    $children = $this->loadTermTree($taxonomy_term->bundle(), $taxonomy_term->id(), 1);
    return new JsonResponse($children);
  }

  /**
   * Loads the taxonomy term tree.
   *
   * @param string $vid
   *   The vocabulary ID.
   * @param int $parent
   *   The parent term ID.
   * @param int $max_depth
   *   The maximum depth to load.
   *
   * @return array
   *   An array of term data.
   */
  protected function loadTermTree($vid, $parent = 0, $max_depth = NULL) {
    $storage = $this->entityTypeManager->getStorage('taxonomy_term');
    $terms = $storage->getQuery()
      ->accessCheck(TRUE)
      ->condition('vid', $vid)
      ->condition('parent', $parent)
      ->sort('weight')
      ->execute();

    $terms = $storage->loadMultiple($terms);
    $tree = [];

    foreach ($terms as $term) {
      $tree[] = [
        'tid' => $term->id(),
        'name' => $term->label(),
        'weight' => (int) $term->weight->value,
        'depth' => $parent === 0 ? 0 : 1,
        'hasChildren' => $this->termHasChildren($term->id()),
      ];
    }

    return $tree;
  }

  /**
   * Checks if a term has children.
   *
   * @param int $tid
   *   The term ID.
   *
   * @return bool
   *   TRUE if the term has children, FALSE otherwise.
   */
  protected function termHasChildren($tid) {
    $count = $this->entityTypeManager->getStorage('taxonomy_term')
      ->getQuery()
      ->accessCheck(TRUE)
      ->condition('parent', $tid)
      ->count()
      ->execute();

    return $count > 0;
  }

  /**
   * Returns a form for editing a taxonomy term via AJAX.
   *
   * @param \Drupal\taxonomy\TermInterface $taxonomy_term
   *   The taxonomy term to edit.
   *
   * @return array
   *   A render array representing the form.
   */
  public function editTermForm(TermInterface $taxonomy_term) {
    $form = \Drupal::service('entity.form_builder')->getForm($taxonomy_term, 'default');

    // Add our custom wrapper for AJAX.
    $form['#prefix'] = '<div id="taxonomy-term-form-wrapper">';
    $form['#suffix'] = '</div>';

    // Add AJAX callback to the submit button.
    $form['actions']['submit']['#ajax'] = [
      'callback' => '::ajaxSubmitForm',
      'wrapper' => 'taxonomy-term-form-wrapper',
    ];

    return $form;
  }

  /**
   * Returns a confirmation form for deleting a taxonomy term via AJAX.
   *
   * @param \Drupal\taxonomy\TermInterface $taxonomy_term
   *   The taxonomy term to delete.
   *
   * @return array
   *   A render array representing the confirmation form.
   */
  public function deleteTermConfirm(TermInterface $taxonomy_term) {
    return \Drupal::service('entity.form_builder')->getForm($taxonomy_term, 'delete');
  }

  /**
   * Returns a form for adding a new taxonomy term via AJAX.
   *
   * @param \Drupal\taxonomy\VocabularyInterface $taxonomy_vocabulary
   *   The vocabulary to add the term to.
   *
   * @return array
   *   A render array representing the form.
   */
  public function addTermForm(VocabularyInterface $taxonomy_vocabulary) {
    return $this->formBuilder->getForm('Drupal\enhanced_taxonomy_manager\Form\TermForm', $taxonomy_vocabulary);
  }

  /**
   * Submit handler for the term form.
   */
  public function termFormSubmit(array &$form, FormStateInterface $form_state) {
    // This method is no longer needed as form handling is done in TermForm class.
  }

  /**
   * AJAX callback for the term form.
   */
  public function ajaxSubmitForm(array $form, FormStateInterface $form_state) {
    // This method is no longer needed as form handling is done in TermForm class.
  }

}
