<?php

namespace Drupal\enhanced_taxonomy_manager\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\taxonomy\VocabularyInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Form for adding taxonomy terms via AJAX.
 */
class TermForm extends FormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new TermForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'taxonomy_term_ajax_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, ?VocabularyInterface $taxonomy_vocabulary = NULL) {
    if (!$taxonomy_vocabulary) {
      throw new NotFoundHttpException('Vocabulary parameter is required.');
    }

    $form['#tree'] = TRUE;
    $form['#attached']['library'][] = 'core/drupal.dialog.ajax';

    $form['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Name'),
      '#maxlength' => 255,
      '#required' => TRUE,
    ];

    // Get parent options.
    $parent_options = ['' => $this->t('- None -')];
    $storage = $this->entityTypeManager->getStorage('taxonomy_term');
    $tree = $storage->loadByProperties(['vid' => $taxonomy_vocabulary->id()]);
    foreach ($tree as $term) {
      $parent_options[$term->id()] = $term->label();
    }

    $form['parent'] = [
      '#type' => 'select',
      '#title' => $this->t('Parent'),
      '#options' => $parent_options,
      '#multiple' => TRUE,
    ];

    $form['vid'] = [
      '#type' => 'value',
      '#value' => $taxonomy_vocabulary->id(),
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#ajax' => [
        'callback' => '::ajaxSubmit',
        'wrapper' => 'taxonomy-term-form-wrapper',
        'effect' => 'fade',
      ],
    ];

    $form['#prefix'] = '<div id="taxonomy-term-form-wrapper">';
    $form['#suffix'] = '</div>';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $term = $this->entityTypeManager->getStorage('taxonomy_term')->create([
      'vid' => $form_state->getValue('vid'),
      'name' => $form_state->getValue('name'),
      'parent' => $form_state->getValue('parent', []),
    ]);

    $term->save();
    $form_state->set('created_term', $term);
  }

  /**
   * AJAX callback for the form submission.
   */
  public function ajaxSubmit(array &$form, FormStateInterface $form_state) {
    if (!$form_state->hasAnyErrors()) {
      $response = new AjaxResponse();
      $response->addCommand(new CloseModalDialogCommand());
      $response->addCommand(new ReplaceCommand(
        '.taxonomy-tree',
        '<div class="taxonomy-tree-refresh-marker"></div>'
      ));
      return $response;
    }
    return $form;
  }

}
